#include <afxwin.h>
#include <process.h>
#include "SMWindow.h"
#include "SMDialog.h"
#include "resource.h"

#define BFCOLOR RGB(255,0,0)
#define HCOLOR  RGB(0,255,0)
#define BMCOLOR RGB(0,0,255)

SMWindow::SMWindow() {
  job_done = false;
  bruteForce_compares = horspool_compares = boyerMoore_compares = 0;
  pattern = text = NULL;
  CRect rect;
  rect.bottom = 512+32; rect.right=512+13;
  rect.top = 50; rect.left = 50;
  Create(NULL,"String Matcher",WS_OVERLAPPEDWINDOW,
	  rect,NULL,NULL);
  
  SMDialog smdiag;
  if(smdiag.DoModal()==-1) exit(0);

  job = smdiag.getJob();
  if(job==NULL) exit(-1);

  runJob();
}

SMWindow::~SMWindow() {
	if(job!=NULL) { 
		if(job->pattern_size==PATTERN_USER_DEFINED) delete job->pattern;
		delete job;
	}
	if(pattern!=NULL) delete pattern;
	if(text!=NULL) delete text;
}

void SMWindow::runJob() {
	// setup pattern and text char *'s
	switch(job->mode) {
	case MODE_BINARY:
		text = "Random binary text";
		if(job->pattern_size==PATTERN_USER_DEFINED)
			pattern = job->pattern;
		else 
			pattern = "Random binary pattern";

		break;
	case MODE_NATURAL:
		switch(job->text) {
		case TEXT_RELNOTES: text = "relnotes.txt"; break;
		case TEXT_REDHAT: text = "redhat.txt"; break;
		case TEXT_EULA: text = "eula.txt"; break;
		case TEXT_WINZIP: text = "winzip.txt"; break;
		case TEXT_DICTIONARY: text = "dictionary.txt"; break;
		default: text = "user.txt"; break;
		}
		if(job->pattern_size==PATTERN_USER_DEFINED)
			pattern = job->pattern;
		else 
			pattern = "Random natural pattern";

		break;
	}

	int num_runs = job->num_runs;
	for(int i=0;i<num_runs;i++) {
		bruteForce_compares += runJob(TYPE_BRUTEFORCE);
		horspool_compares += runJob(TYPE_HORSPOOL);
		boyerMoore_compares += runJob(TYPE_BOYERMOORE);
	}

	bruteForce_compares = bruteForce_compares / num_runs;
	horspool_compares = horspool_compares / num_runs;
	boyerMoore_compares = boyerMoore_compares / num_runs;

//	MessageBox(itoa(bruteForce_compares,new char[16],10));
//	MessageBox(itoa(horspool_compares,new char[16],10));
//	MessageBox(itoa(boyerMoore_compares,new char[16],10));
	job_done = true;
}

long SMWindow::runJob(int type) {

	char * executable;
	char * operation_mode;
	char * arg1, *arg2;

	// determining executable
	switch(type) {
	case TYPE_BOYERMOORE:
		executable = "boyerMoore.exe";
		break;
	case TYPE_HORSPOOL:
		executable = "horspool.exe";
		break;
	case TYPE_BRUTEFORCE:
	default:
		executable = "bruteForce.exe";
		break;
	}

	// determining operation mode
	switch(job->mode) {
	case MODE_BINARY:
		operation_mode = "b";
		break;
	case MODE_NATURAL:
		operation_mode = "s";
		break;
	}
	
	
	int i = 0;
	// determining arg1
	switch(job->mode) {
	case MODE_BINARY:
		arg1 = new char[8];
		for(i=0;i<8;i++) arg1[i]=NULL;
		itoa(job->text,arg1,10);
		break;
	case MODE_NATURAL:
		switch(job->text) {
		case TEXT_RELNOTES: arg1 = "relnotes.txt"; break;
		case TEXT_REDHAT: arg1 = "reghat.txt"; break;
		case TEXT_EULA: arg1 = "eula.txt"; break;
		case TEXT_DICTIONARY: arg1 = "dictionary.txt"; break;
		case TEXT_WINZIP: arg1 = "winzip.txt"; break;
		case TEXT_USER:
		default: arg1 = "user.txt"; break;
		}
	}

	// determining arg2
	switch(job->mode) {
	case MODE_BINARY:
		arg2 = new char[8];
		for(i=0;i<8;i++) arg2[i]=NULL;
		itoa(job->pattern_size,arg2,10);
		break;
	case MODE_NATURAL:
		if(job->pattern_size==PATTERN_USER_DEFINED) {
			FILE * tempfile = fopen("doJobTempFile2","w");
			if(tempfile==NULL) {
				MessageBox("Can't open temp file",NULL,MB_ICONERROR);
				exit(-1);
			}
			for(i=0;i<256 && job->pattern[i]!=NULL;i++)
				fputc(job->pattern[i],tempfile);
			fclose(tempfile);

			arg2 = "doJobTempFile2";
		} else {
			arg2 = new char[8];
			for(i=0;i<8;i++) arg2[i]=NULL;
			itoa(job->pattern_size,arg2,10);
			break;
		}

		break;
	}

	// args set up...do spawn
//	MessageBox(executable);
//	MessageBox(operation_mode);
//	MessageBox(arg1);
//	MessageBox(arg2);
	if(spawnlp(P_WAIT,executable,executable,operation_mode,arg1,arg2,
		"doJobTempFile1",NULL)!=0) {
		MessageBox("A child process failed to exit with code 0.",
					"Major, Life-ending error.",
					MB_ICONERROR);
		MessageBox(executable);
		exit(-1);
	}

	// now retreive output from executable's output file
	FILE * infile = fopen("doJobTempFile1","r");
	if(infile==NULL) {
		MessageBox("Unable to open results file.",NULL,MB_ICONERROR);
		exit(-1);
	}

	char line_in[256];
	fgets(line_in,256,infile);
	fclose(infile);


	unlink("doJobTempFile1");
	unlink("doJobTempFile2");

	return atoi(line_in);
}

afx_msg void SMWindow::OnPaint() {

	if(!job_done) return;

	CPaintDC dc(this);
	
	CRect window;
	GetClientRect(window);

	// setup the axis
	dc.MoveTo(5,30);
	dc.LineTo(5,window.bottom-5);
	dc.MoveTo(6,window.bottom-5);
	dc.LineTo(6,30);
	dc.MoveTo(5,window.bottom-5);
	dc.LineTo(window.right-5,window.bottom-5);
	dc.MoveTo(5,window.bottom-5-1);
	dc.LineTo(window.right-5,window.bottom-5-1);

	dc.FillSolidRect(5,3,20,20,BFCOLOR);
	dc.FillSolidRect(5+150,3,20,20,HCOLOR);
	dc.FillSolidRect(5+300,3,20,20,BMCOLOR);

	dc.FillSolidRect(100,100,1,1,RGB(255,255,255)); // return to white background

	dc.TextOut(5+20+2,3,"Brute Force");
	dc.TextOut(5+150+20,3,"Horspool");
	dc.TextOut(5+300+20,3,"Boyer-Moore");

	int barDivisor;

	if(bruteForce_compares > horspool_compares) {
		if(bruteForce_compares > boyerMoore_compares) {
			// bf max
			barDivisor = (int) (bruteForce_compares/((long) 400));
		} else {
			// bm max
			barDivisor = (int) (boyerMoore_compares/((long) 400));
		}
	} else {
		if(horspool_compares > boyerMoore_compares) {
			// ho max
			barDivisor = (int) (horspool_compares/((long) 400));
		} else {
			// 
			if(bruteForce_compares > horspool_compares) {
				// bf Max
				barDivisor = (int) (bruteForce_compares/((long) 400));
			} else {
				// ho Max
				barDivisor = (int) (horspool_compares/((long) 400));
			}

		}
	}

	// code to make colored bars

	int bf_height, bm_height, h_height;

	h_height = horspool_compares/barDivisor;
	bm_height = boyerMoore_compares/barDivisor;
	bf_height = bruteForce_compares/barDivisor;

	dc.FillSolidRect(window.right/10,window.bottom-6-bf_height,window.right/5,bf_height,BFCOLOR);
    dc.FillSolidRect(2*window.right/10+window.right/5,window.bottom-6-h_height,window.right/5,h_height,HCOLOR);
    dc.FillSolidRect(3*window.right/10+2*window.right/5,window.bottom-6-bm_height,window.right/5,bm_height,BMCOLOR);

	//  code here to do height labeling
	dc.FillSolidRect(1,1,1,1,RGB(255,255,255)); // back to white background

	dc.TextOut(window.right/10,window.bottom-bf_height-22,itoa(bruteForce_compares,new char[8],10));
	dc.TextOut(2*window.right/10+window.right/5,window.bottom-h_height-22,itoa(horspool_compares,new char[8],10));
	dc.TextOut(3*window.right/10+2*window.right/5,window.bottom-bm_height-22,itoa(boyerMoore_compares,new char[8],10));

}  


BEGIN_MESSAGE_MAP( SMWindow, CFrameWnd )
  ON_WM_PAINT()
END_MESSAGE_MAP()

class StringMatchingApp : public CWinApp {
public:
	BOOL InitInstance() {
		m_pMainWnd = new SMWindow;
		m_pMainWnd->ShowWindow( m_nCmdShow );
		m_pMainWnd->UpdateWindow();
		return TRUE;
	}
} stringmatcherapp;